##########################
### ROZDZIAL 1 ###########
##########################
### Zmienna licznikowa ###
##########################
### (C) Ewelina Chmura ###
##########################

# Instalacja pakietow
pakiety <- c("AER", "ggplot2", "MASS", "plyr", "pscl")
install.packages(pakiety)
lapply(pakiety, require, character.only = TRUE)

# Podaj sciezke do przestrzeni roboczej
setwd("C:/Andrzej/OneDrive - SGH/Dydaktyka/ksiazka_kolo/zastosowania_ekonometrii_www/01")

# Import danych
dane <- read.csv2("rozdzial_1.csv", dec=",", stringsAsFactors = FALSE)

# Eksploracja danych
names(dane)
str(dane)
hist(dane$scores)

# Wykres 1.1 - rozklad liczby bramek 
plot(table(dane$scores))
table(dane$scores)

# Srednia i wariancja liczba goli zdobytych przez pojedyncza druzyne
theta <- round(mean(dane$scores), 3) # = ENW(theta)
theta
var(dane$scores)

# Tabela 1.1 - dopasowany rozkald Poissona
tabela_1 <- data.frame(scores = 0:max(dane$scores))
df_temp <- as.data.frame(table(dane$scores))  
tabela_1 <- merge(x = tabela_1, y = df_temp, by.x = "scores", by.y = colnames(df_temp)[1], all.x = TRUE)
tabela_1[is.na(tabela_1)] <- 0
names(tabela_1) <- c("scores", "num")
tabela_1$num_poiss <- round((((theta ^ tabela_1$scores) * exp(- theta)) / factorial(tabela_1$scores)) * nrow(dane), 2)
tabela_1
rm(df_temp)

# Wykres 1.2 - dopasowany rozkald Poissona
barplot(t(as.matrix(tabela_1[, 2:3])), beside = TRUE, ylim = c(0, 50), names = as.character(tabela_1$scores))
legend("topright", legend = c("empiryczny", "dopasowany r. Poissona"), cex = 0.7, pch = 1, pt.cex = 1)

# Generowanie i transformacja zmiennych
dane$time <- ifelse(dane$time == 90, 0, 1)
dane$distance_ratio <- dane$distance / dane$distance_o
dane$recovered_balls_ratio <- dane$recovered_balls / dane$recovered_balls_o
dane$lost_balls_ratio <- dane$lost_balls / dane$lost_balls_o

dane.do.modelu <- dane[, - which(names(dane) %in% c("date", "team", "opponent", "stage", "round"))]

# Model Poissona
model.poiss <- glm(scores ~ on_target + time + FIFA_rank + distance_ratio
                        + total_attempts_o + red_card + recovered_balls_ratio, 
                        family = "poisson", data = dane.do.modelu)
summary(model.poiss)

# Wartosci dopasowane przez model Poissona
df_prognozy <- as.data.frame(cbind(dane[, c(1:3, 6)]))
df_prognozy$model_poiss <- round(predict(model.poiss, type = "response")) # dla proby modelowej rowniez model.poiss$fitted.values
df_prognozy

# Wykres obserwacji: empiryczne vs. modelowe
df_wykres_emp <- data.frame(type = "empirical", scores = df_prognozy$scores)
df_wykres_model <- data.frame(type = "model_poisson", scores = df_prognozy$model_poiss)
df_wykres <- rbind(df_wykres_emp, df_wykres_model)
  
ggplot(df_wykres, aes(x = scores, fill = type)) + geom_bar(position = "dodge")

tabela_emp_model <- merge(x = ddply(df_prognozy, .(scores), nrow), 
                          y = ddply(df_prognozy, .(model_poiss), nrow), 
                          by.x = "scores",
                          by.y = "model_poiss",
                          all = TRUE)
tabela_emp_model[is.na(tabela_emp_model)] <- 0
colnames(tabela_emp_model) <- c("scores", "empiryczny", "model_poissona")
tabela_emp_model

# Trafnosc prognoz
sum(df_prognozy$scores == df_prognozy$model_poiss) / nrow(df_prognozy)

# Test na nadmierne rozproszenie
dispersiontest(model.poiss, trafo = 2, alternative = "greater")
dispersiontest(model.poiss, trafo = 2, alternative = "two.sided")
dispersiontest(model.poiss, trafo = 2, alternative = "less")

# Hurdle
# - porownanie liczby obserwacji z wartoscia 0: empiryczne vs. modelowe
sum(df_prognozy$scores == 0)
sum(df_prognozy$model_poiss == 0)

model.hurdle <- hurdle(scores ~ time + FIFA_rank + on_target + distance_ratio
                                | time + completion_rate + on_target + free_kicks + red_card +
                                no_matches_before_WC + before_WC_performance +
                                FIFA_rank_o + completion_rate_o + corners_o + lost_balls_ratio, 
                       data = dane.do.modelu, dist = "poisson")
summary(model.hurdle)

df_prognozy$model_hurdle <- round(predict(model.hurdle, type = "response")) 
df_prognozy

sum(df_prognozy$scores == df_prognozy$model_hurdle) / nrow(df_prognozy)

# Wykres 1.3
df_wykres_model_h <- data.frame(type = "model_hurdle", scores = df_prognozy$model_hurdle)
df_wykres <- rbind(df_wykres, df_wykres_model_h)
ggplot(df_wykres, aes(x = scores, fill = type)) + geom_bar(position = "dodge")

tabela_emp_model <- merge(x = tabela_emp_model, 
                          y = ddply(df_prognozy, .(model_hurdle), nrow), 
                          by.x = "scores",
                          by.y = "model_hurdle",
                          all = TRUE)
tabela_emp_model[is.na(tabela_emp_model)] <- 0
colnames(tabela_emp_model)[4] <- "model_hurdle"
tabela_emp_model

# Negative binomial
model.negbin <- glm.nb(scores ~ on_target + time + FIFA_rank + distance_ratio
                       + total_attempts_o + red_card + recovered_balls_ratio ,data = dane.do.modelu)
summary(model.negbin)