###################################################
### ROZDZIAL 9 ####################################
###################################################
### Regresja liniowa - podejcie bayesowskie ######
###################################################
### (C) Bartosz Olesiski #########################
###################################################

################################################################################################

##Skrypt 1

install.packages("mvtnorm")
install.packages("lmtest")
install.packages("coda")


################################################################################################

##Skrypt 2

rm(list = ls()) #Oczyszczenie przestrzeni roboczej
setwd("C:/zastosowania_ekonometrii_www/09/")
dane <- read.csv2("rozdzial_9.csv", header = TRUE, sep = ",", dec = ".")
t(summary(dane)) #rednia, minimum, maksimum
sqrt(diag(cov(dane))) / apply(dane, 2, mean) #Wspczynniki zmiennoci


################################################################################################

##Skrypt 3

library(lmtest) #Pakiet zawierajcy test Breuscha-Pagana (Zeileis, Hothorn, 2002).
model1 <- lm(cena ~ produkcja + pojemnosc + moc + przebieg + bezwypadkowy + wlasciciel + paliwo + skrzynia, data = dane)
summary(model1)
bptest(model1)
model2 <- lm(cena ~ produkcja + moc + przebieg + bezwypadkowy + wlasciciel + paliwo + skrzynia, data = dane)
summary(model2)
bptest(model2)
model3 <- lm(log(cena) ~ produkcja + log(moc) + log(przebieg) + bezwypadkowy + wlasciciel + paliwo + skrzynia, data = dane)
summary(model3)
bptest(model3)


################################################################################################

##Skrypt 4

korelacja.poziom <- cor(cbind(dane$cena, dane$produkcja, dane$pojemnosc, dane$moc, dane$przebieg))
rownames(korelacja.poziom) <- colnames(korelacja.poziom) <- c("cena", "produkcja", "pojemnosc", "moc", "przebieg")
korelacja.poziom

################################################################################################

##Skrypt 5

korelacja.log <- cor(cbind(log(dane$cena), dane$produkcja, log(dane$moc), log(dane$przebieg)))
rownames(korelacja.log) <- colnames(korelacja.log) <- c("ln(cena)", "produkcja", "ln(moc)", "ln(przebieg)")
korelacja.log

################################################################################################

##Skrypt 6

ols <- lm(log(cena) ~ produkcja + log(moc) + log(przebieg) + bezwypadkowy + wlasciciel + paliwo + skrzynia, data = dane)
ols.beta <- ols$coefficients
ols.sigma <- vcov(ols)


################################################################################################

##Skrypt 7

library(mvtnorm) #Pakiet zawierajcy funkcje dla wielowymiarowego rozkadu t-Studenta (Genz et al., 2012)
S <- 10^5
v <- nrow(dane) - length(ols.beta)
set.seed(1)
wszystkie.beta <- rmvt(S, sigma = ols.sigma, df = v, delta = ols.beta, type = "shifted")


################################################################################################

##Skrypt 8

bayes.beta1 <- apply(wszystkie.beta, 2, mean)
bayes.sigma1 <- cov(wszystkie.beta)
bayes.wyniki1 <- cbind(bayes.beta1,
                       sqrt(diag(bayes.sigma1)),
                       bayes.beta1 / sqrt(diag(bayes.sigma1)))
rownames(bayes.wyniki1) <- names(ols.beta)
colnames(bayes.wyniki1) <- c("Estimate", "Std. Error", "t value")
summary(ols)
bayes.wyniki1


################################################################################################

##Skrypt 9

b1 <- wszystkie.beta[, 1] # staa
b2 <- wszystkie.beta[, 2] # produkcja
b3 <- wszystkie.beta[, 3] # ln(moc)
b4 <- wszystkie.beta[, 4] # ln(przebieg)
b5 <- wszystkie.beta[, 5] # bezwypadkowy
b6 <- wszystkie.beta[, 6] # wlasciciel
b7 <- wszystkie.beta[, 7] # paliwo
b8 <- wszystkie.beta[, 8] # skrzynia
beta.uciete <- wszystkie.beta[b2>0 & b3>0 & b4<0 & b5>0 & b6>0 & b7>0 & b8>0, ]


################################################################################################

##Skrypt 10

prob <- nrow(beta.uciete) / nrow(wszystkie.beta)
prob


################################################################################################

##Skrypt 11

uciete.b8 <- beta.uciete[,8] # skrzynia
bandwidth <- 0.003
dziedzina <- density(b8, bw = bandwidth, n = 1024, from = -0.3, to = 0.3)$x
dziedzina.ucieta <- density(c(uciete.b8, -uciete.b8), from = 0, to = 0.3, bw = bandwidth, n = 512)$x
gestosc <-density(b8, bw = bandwidth, n = 1024,from = -0.3, to = 0.3)$y
gestosc <- gestosc/sum(gestosc)
gestosc.ucieta <- density(c(uciete.b8, -uciete.b8), from = 0, to = 0.3, bw = bandwidth, n = 512)$y
gestosc.ucieta <- gestosc.ucieta / sum(gestosc.ucieta)
plot(dziedzina.ucieta, gestosc.ucieta, type = "l",
     xlim = c(-0.3, 0.3), ylim = c(0, max(gestosc.ucieta)))
lines(dziedzina, gestosc, type = "l", col = "red")
abline(v = 0, col = "black")
#Zapisanie danych do wykresu 9.1 do pliku wykres_9_1.csv
wykres <- cbind(dziedzina, gestosc, c(rep(0, 512), gestosc.ucieta))
colnames(wykres) <- c("Beta_8", "Gsto nieograniczona", "Gsto ucita")
write.table(wykres, file = "wykres_9_1.csv", dec = ".", sep = ";", row.names = FALSE)

################################################################################################

##Skrypt 12

library(coda) #Pakiet sucy do wyliczenia przedziaw HPD (Plummer et al., 2006)
bayes.beta2 <- apply(beta.uciete, 2, mean)
przedzialy.hpd.nieuciete <- HPDinterval(mcmc(wszystkie.beta)) #funkcja liczca bayesowskie przedziay HPD (95%)
przedzialy.hpd.uciete <- HPDinterval(mcmc(beta.uciete)) 
bayes.wyniki2 <- cbind(bayes.beta1, przedzialy.hpd.nieuciete,
                       bayes.beta2, przedzialy.hpd.uciete)
rownames(bayes.wyniki2) <- names(ols.beta)
colnames(bayes.wyniki2) <- c("Oszacowania nieucite", "Dolny HPD", "Grny HPD",
                             "Oszacowania ucite", "Dolny HPD", "Grny HPD")
bayes.wyniki2
                             

################################################################################################


##Skrypt 13

#Przygotowanie przestrzeni roboczej i danych
rm(list = ls()) #Oczyszczenie przestrzeni roboczej
setwd("C:/zastosowania_ekonometrii_www/09/")
dane <- read.csv2("rozdzial_9.csv", header = TRUE, sep = ",", dec = ".")
library(mvtnorm)
colnames(dane)
dane_short <- dane[, -3]
colnames(dane_short)

# Dugoci prby testowej
O <- c(1, 5, 10, 15, 20, 25, 30, 35, 40)
# Ile razy maksymalnie dzielimy nasz prb (losowo)
K_max <- 1000

#Ile razy losujemy z rozkadu a posteriori
S <- 10^5
final.results <- matrix(NA, length(O), 5)
rownames(final.results) <- O
colnames(final.results) <- c("OLS RMSE", "OLS Calculation Errors", "Bayesian RMSE", "Bayesian Calculation Errors", "Probability")
set.seed(1)
iteracja <- 0

#Nastpny fragment kodu (podwjna ptla) jest do dugi, prosimy nie panikowa :)
for(i in 1:length(O)){
  o <- O[i]
  
  #Na ile sposobw dzielimy prb:
  K <- min(choose(nrow(dane_short), o), K_max)
  forecast.errors <- matrix(NA, K, 5)
    for(k in 1:K){
      
      #Liczymy iteracj
      iteracja <- iteracja + 1
      print(paste("Iteracja ", iteracja, " spord ", (length(O) - 1) * K_max + nrow(dane_short), ".", sep = ""))
      outofsample <- dane_short[sample(1:nrow(dane_short), o), ]
      insample <- dane_short[-as.numeric(rownames(outofsample)), ]
      y_empiryczne <- outofsample$cena
  
      #Ucinamy zmienna objasniana
      outofsample <- outofsample[, -1]
      outofsample <- cbind(1, as.matrix(outofsample))
      ols <- lm(log(cena) ~ produkcja + log(moc) + log(przebieg) + bezwypadkowy + wlasciciel + paliwo + skrzynia, data = insample)
      ols.beta <- ols$coefficients
      ols.sigma <- vcov(ols)
      v <- nrow(insample) - ncol(insample)

      #Przyblienie Monte Carlo - generujemy wektory oszacowan z rozkladu a posteriori
      #bez restrykcji
      #Rzadki przypadek - wylosuje si tak, e ktra ze zmiennych binarnych bdzie wspliniowa.
      wszystkie.beta <- tryCatch(rmvt(S, sigma = ols.sigma, df = v, delta = ols.beta, type = "shifted"),
                                 error = function(e) matrix(NA, S, ncol(insample)))
      
      #shifted daje odpowiedni rozklad dla symulacji z rozkladu a posteriori
      b1 <- wszystkie.beta[, 1] # staa
      b2 <- wszystkie.beta[, 2] # produkcja
      b3 <- wszystkie.beta[, 3] # ln(moc)
      b4 <- wszystkie.beta[, 4] # ln(przebieg)
      b5 <- wszystkie.beta[, 5] # bezwypadkowy
      b6 <- wszystkie.beta[, 6] # wlasciciel
      b7 <- wszystkie.beta[, 7] # paliwo
      b8 <- wszystkie.beta[, 8] # skrzynia
      beta.uciete <- wszystkie.beta[b2>0 & b3>0 & b4<0 & b5>0 & b6>0 & b7>0 & b8>0, ]
      #Czasami adne beta nie speniaj restrykcji. Musimy mimo to zagwarantowa dalszy przebieg ptli.
      bayes.beta <- tryCatch(apply(beta.uciete, 2, mean), error = function(e) rep(NA, ncol(insample)))
      bayes.prob <- nrow(beta.uciete) / nrow(wszystkie.beta)

      #Liczymy prognozy
      #Doliczamy logarytmy dla prby testowej
      outofsample[, 3:4] <- log(outofsample[, 3:4])
      ols.predict <- outofsample %*% ols.beta
      bayes.predict <- outofsample %*% bayes.beta
      rmse.ols <- sqrt(mean((y_empiryczne - exp(ols.predict)) ^ 2, na.rm = TRUE))
      rmse.bayes <- sqrt(mean((y_empiryczne - exp(bayes.predict)) ^ 2, na.rm = TRUE))

      #Ile razy ptla musiaa by puszczona pomimo bdu
      errors.ols <- sum(is.na(ols.predict))
      errors.bayes <- sum(is.na(bayes.predict))
 
      #rednie prawdopodobiestwo a posteriori spenienia wszystkich restrykcji
      prob.bayes <- mean(bayes.prob, na.rm = TRUE)
      
      #Wypeniamy tabelk ze rednimi bdami prognoz dla kadej wylosowanej prby testowej
      forecast.errors[k, ] <- c(rmse.ols, errors.ols, rmse.bayes, errors.bayes, prob.bayes)
      }
  #Uredniamy wyniki tak, by otrzyma sumaryczne miary dla kadej wielkoci prby testowej
  final.results[i, ] <- apply(forecast.errors, 2, mean, na.rm = TRUE)
}
rmse.ratio <- final.results[, 3] / final.results[, 1]
final.results <- cbind(final.results, rmse.ratio)
colnames(final.results)[6] <- "RMSE ratio"
final.results

################################################################################################

#Dodatek - porwnanie predykcji z rozkadu predykcyjnego a posteriori 
#z prost prognoz z wiczenia wyej dla modelu bayesowskiego z restrykcjami i bez

#Przygotowanie przestrzeni roboczej i danych
rm(list = ls()) #Oczyszczenie przestrzeni roboczej
setwd("C:/zastosowania_ekonometrii_www/09/")
dane <- read.csv2("rozdzial_9.csv", header = TRUE, sep = ",", dec = ".")
library(mvtnorm)
colnames(dane)
dane_short <- dane[, -3]
colnames(dane_short)
outofsample <- dane_short[1, ]
insample <- dane_short[-1, ]
S <- 10^6

#Ucinamy zmienna objasniana
outofsample <- outofsample[, -1]
outofsample <- cbind(1, as.matrix(outofsample))
outofsample[, 3:4] <- log(outofsample[, 3:4])
ols <- lm(log(cena) ~ produkcja + log(moc) + log(przebieg) + bezwypadkowy + wlasciciel + paliwo + skrzynia, data = insample)
ols.beta <- ols$coefficients
ols.sigma <- vcov(ols)
v <- nrow(insample)-ncol(insample)

#Przyblienie Monte Carlo - generujemy wektory oszacowan z rozkladu a posteriori
#bez restrykcji
set.seed(1)
wszystkie.beta <- rmvt(S, sigma = ols.sigma, df = v, delta = ols.beta, type = "shifted")
#Rozkad predyktywny a posteriori - na razie przypadek bez restrykcji

#Przypadek 1: rozkad predyktywny niewarunkowany wzgldem beta
bayes.beta.full <- apply(wszystkie.beta, 2, mean)
y <- as.matrix(log(insample[, 1]))
X <- cbind(1, as.matrix(insample[, -1]))
X[, 3:4] <- log(X[, 3:4])
V <- solve(t(X) %*% X)
epsilon <- y - X %*% bayes.beta.full
s_2 <- t(epsilon) %*% epsilon / v
#beta.check <- solve(t(X) %*% X) %*% t(X) %*% y
predictive.distribution.1 <- rmvt(S, sigma = s_2*(1 + outofsample %*% V %*% t(outofsample)), df = v,
                                delta = outofsample %*% bayes.beta.full,
                                type = "shifted")

#Przypadek 2: rozkad predytkywny warunkowany wzgldem beta 
predictive.draw <- function(beta.row){
  ret <- rmvt(1, sigma = s_2, df = v, delta = outofsample %*% beta.row, type = "shifted")
  ret
}
predictive.distribution.2 <- apply(wszystkie.beta, 1, predictive.draw) #Wykorzystujemy dokadnie te same beta

#shifted daje odpowiedni rozklad dla symulacji z rozkladu a posteriori
b1 <- wszystkie.beta[, 1] # staa
b2 <- wszystkie.beta[, 2] # produkcja
b3 <- wszystkie.beta[, 3] # ln(moc)
b4 <- wszystkie.beta[, 4] # ln(przebieg)
b5 <- wszystkie.beta[, 5] # bezwypadkowy
b6 <- wszystkie.beta[, 6] # wlasciciel
b7 <- wszystkie.beta[, 7] # paliwo
b8 <- wszystkie.beta[, 8] # skrzynia
beta.uciete <- wszystkie.beta[b2>0 & b3>0 & b4<0 & b5>0 & b6>0 & b7>0 & b8>0, ]

#Przypadek 3: rozkad predyktywny niewarunkowany wzgldem beta
bayes.beta.uciete <- apply(beta.uciete, 2, mean)
epsilon_uciete <- y - X %*% bayes.beta.uciete
s_2_uciete <- t(epsilon_uciete) %*% epsilon_uciete / v
predictive.distribution.3 <- rmvt(S, sigma = s_2_uciete * (1 + outofsample %*% V %*% t(outofsample)), df = v,
                                  delta = outofsample %*% bayes.beta.uciete,
                                  type = "shifted")

#Przypadek 4: rozkad predytkywny warunkowany wzgldem beta 
predictive.distribution.4 <- apply(beta.uciete, 1, predictive.draw) #Wykorzystujemy ten sam zestaw beta, ale ucity

bandwidth <- 0.003
plot(density(predictive.distribution.1, bw=bandwidth, n=1024, from = 10, to = 12), main = "Gstoci predykcyjne")
lines(density(predictive.distribution.2, bw=bandwidth, n=1024, from = 10, to = 12), type = "l", col = "red")
lines(density(predictive.distribution.3, bw=bandwidth, n=1024, from = 10, to = 12), type = "l", col = "green")
lines(density(predictive.distribution.4, bw=bandwidth, n=1024, from = 10, to = 12), type = "l", col = "blue")