###################################################
### ROZDZIAL 10 ###################################
###################################################
### Model (S)VAR ##################################
###################################################
### (C) Micha Chojnowski #########################
###################################################

### BIBLIO ###

install.packages("vars")
install.packages("astsa")
install.packages("graphics")
install.packages("plotrix")
install.packages("tseries")
install.packages("matrixcalc")

library(vars)
library(astsa)
library(graphics)
library(plotrix)
library(tseries)
library(matrixcalc)

### HEAD ###
rm(list = ls(all = TRUE))
setwd("C:/zastosowania_ekonometrii_www/10/")
dev.new()
op <- par(no.readonly = TRUE)
dev.off
par(op)

###BODY###
#Wczytanie danych
dane <- read.csv("rozdzial_10.csv", sep = ";", dec = ".")[-1]
plot(ts(dane[, 2], start = c(1991, 1), freq = 12), type="l", ylab = "Aktywno", xlim = c(1990, 2016), xlab = "Rok")

#Okrelenie stopnia zintegrowania szeregw
n <- length(dane[1, ])
output <- array(dim = c(n, 3))
for (i in 1:n){
	output[i, 1] <- colnames(dane)[i]
	output[i, 2] <- round(adf.test(dane[, i])$p.value, 3)
	output[i, 3] <- round(kpss.test(dane[, i])$p.value, 3)
}
output
for (i in 1:n){
	dane[, i] <- c(NA, diff(dane[, i]))
}	
dane <- na.omit(dane)
output <- array(dim = c(n, 3))
for (i in 1:n){
	output[i, 1]<-colnames(dane)[i]
	output[i, 2]<-round(adf.test(dane[, i])$p.value, 3)
	output[i, 3]<-round(kpss.test(dane[, i])$p.value, 3)
}
output
rm(output)

#Okrelenie optymalnej liczby opnie
VARselect(dane, lag.max = 6, type = "both")
model_alt <- VAR(dane, p = 2)
model_zer <- VAR(dane, p = 1)
D <- (- 2 * logLik(model_zer) + 2 * logLik(model_alt))
pchisq(D, (model_alt$K ^ 2 * model_alt$p + model_alt$K) - (model_zer$K ^ 2 * model_zer$p + model_zer$K))[1]
rm(D, model_alt, model_zer)

#Oszacowanie mdoelu VAR(1)
model <- VAR(dane, p = 1)
Acoef(model)
coef(model)

#Okreslenie stacjonarnoci modelu poprzez macierz towarzyszc
macierzA <- Bcoef(model)[, -length(Bcoef(model)[1, ])]
macierzA_dol <- cbind(diag((model$p - 1) * n), array(0, dim = c((model$p - 1) * n, n)))
macierzA <- rbind(macierzA, macierzA_dol)
wartosci <- eigen(macierzA)$values
plot(Re(wartosci), Im(wartosci), 
     xlim = c(min(Re(wartosci), -1.2), max(Re(wartosci), 1.2)), 
     ylim = c(min(Im(wartosci), -1.2), max(Im(wartosci), 1.2)), 
     pch = 21, col = "red", bg = "orange", xlab = "", ylab = "", 
     main = "Wartosci wlasne macierzy towarzyszacej")
draw.circle(0, 0, 1)
abline(h = 0, col = "808080")
rm(macierzA, macierzA_dol, wartosci)

#t-test dla reszt
vres <- residuals(model)
par(mfrow = c(1, 3), mar = c(0, 4, 0, 0))
hist(vres[, 1], main = NULL, xlab = "reszty rwnania 1", ylab = "czsto")
hist(vres[, 2], main = NULL, xlab = "reszty rwnania 2", ylab = "czsto")
hist(vres[, 3], main = NULL, xlab = "reszty rwnania 3", ylab = "czsto")
write.csv(vres, "reszty_wykres.csv")

#testy dla reszt modelu VAR(1)
serial.test(model, lags.pt = 16, type = "PT.asymptotic")
serial.test(model, lags.pt = 16, type = "BG")
normality.test(model)

#testy dla reszt modelu VAR(2)
model <- VAR(dane,p=2)
serial.test(model, lags.pt = 16, type = "PT.asymptotic")
serial.test(model, lags.pt = 16, type = "BG")
normality.test(model)
model <- VAR(dane, p = 6)
serial.test(model, lags.pt = 16, type = "PT.asymptotic")
serial.test(model, lags.pt = 16, type = "BG")
normality.test(model)

#Zmiana modelu na VAR(2)
model<-VAR(dane, p = 2)

#Analiza modelu VAR
predict(model, n.ahead = 6)
plot(predict(model,n.ahead = 6), xlim = c(length(dane[, 1])-30, length(dane[, 1]) + 7))
plot(irf(model, n.ahead = 24))
fevd(model, n.ahead = 5)

#Przyczynowo w sensie Grangera
causality(model, cause = "AKTYWNOSC")

#analia modelu SVAR
A0 <- diag(3)
A0[1:3, 1] <- NA
A0[2:3, 2] <- NA
A0[3, 3] <- NA
model_svar <- SVAR(model, Bmat = A0)
model_svar
plot(irf(model_svar, n.ahead = 24))
rm(A0)

#korelacja
model_svar$B
pure_res <- t(model_svar$B %*% t(residuals(model)))

###TAIL###
rm(model, model_svar, dane, i, n)
ls()
